@@ -20,6 +20,7 @@
  ***************************************************************************/
 
 #include "PackageKitBackend.h"
+#include "PackageKitSourcesBackend.h"
 #include "PackageKitResource.h"
 #include "PackageKitUpdater.h"
 #include "AppPackageKitResource.h"
@@ -27,11 +28,14 @@
 #include "AppstreamReviews.h"
 #include <resources/AbstractResource.h>
 #include <resources/StandardBackendUpdater.h>
+#include <resources/SourcesModel.h>
 #include <Transaction/TransactionModel.h>
+
 #include <QStringList>
 #include <QDebug>
 #include <QTimer>
 #include <QTimerEvent>
+
 #include <PackageKit/Transaction>
 #include <PackageKit/Daemon>
 #include <PackageKit/Details>
@@ -49,9 +53,14 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     , m_reviews(new AppstreamReviews(this))
 {
     bool b = m_appdata.open();
-    Q_ASSERT(b && "must be able to open the appstream database");
     if (!b) {
         qWarning() << "Couldn't open the AppStream database";
+
+        auto msg = new QAction(i18n("Got it"), this);
+        msg->setWhatsThis(i18n("Please make sure that Appstream is properly set up on your system"));
+        msg->setPriority(QAction::HighPriority);
+        connect(msg, &QAction::triggered, msg, [msg](){ msg->setVisible(false); });
+        m_messageActions << msg;
     }
     reloadPackageList();
 
@@ -65,11 +74,17 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     updateAction->setIcon(QIcon::fromTheme(QStringLiteral("system-software-update")));
     updateAction->setText(i18nc("@action Checks the Internet for updates", "Check for Updates"));
     updateAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
+    connect(this, &PackageKitBackend::fetchingChanged, updateAction, [updateAction, this](){
+        updateAction->setEnabled(!isFetching());
+    });
     connect(updateAction, &QAction::triggered, this, &PackageKitBackend::refreshDatabase);
     m_messageActions += updateAction;
 
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::updatesChanged, this, &PackageKitBackend::fetchUpdates);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::isRunningChanged, this, &PackageKitBackend::checkDaemonRunning);
+    connect(m_reviews, &AppstreamReviews::ratingsReady, this, &AbstractResourcesBackend::emitRatingsReady);
+
+    SourcesModel::global()->addSourcesBackend(new PackageKitSourcesBackend(this));
 }
 
 PackageKitBackend::~PackageKitBackend()
@@ -88,11 +103,6 @@ void PackageKitBackend::acquireFetching(bool f)
     else
         m_isFetching--;
 
-    if (m_isFetching==0) {
-        m_packages = m_updatingPackages;
-        m_updatingPackages.clear();
-    }
-
     if ((!f && m_isFetching==0) || (f && m_isFetching==1)) {
         emit fetchingChanged();
     }
@@ -101,34 +111,39 @@ void PackageKitBackend::acquireFetching(bool f)
 
 void PackageKitBackend::reloadPackageList()
 {
-    m_updatingPackages = m_packages;
-    
+    acquireFetching(true);
     if (m_refresher) {
         disconnect(m_refresher.data(), &PackageKit::Transaction::finished, this, &PackageKitBackend::reloadPackageList);
     }
 
-    foreach(const Appstream::Component& component, m_appdata.allComponents()) {
+    const auto components = m_appdata.allComponents();
+    QStringList neededPackages;
+    neededPackages.reserve(components.size());
+    foreach(const Appstream::Component& component, components) {
         if (component.packageNames().isEmpty()) {
             qDebug() << "no packages for" << component.name();
             continue;
         }
+        neededPackages += component.packageNames();
 
         const auto res = new AppPackageKitResource(component, this);
-        m_updatingPackages.packages[component.id()] = res;
+        m_packages.packages[component.id()] = res;
         foreach (const QString& pkg, component.packageNames()) {
-            m_updatingPackages.packageToApp[pkg] += component.id();
+            m_packages.packageToApp[pkg] += component.id();
         }
 
         foreach (const QString& pkg, component.extends()) {
-            m_updatingPackages.extendedBy[pkg] += res;
+            m_packages.extendedBy[pkg] += res;
         }
     }
+    acquireFetching(false);
+    neededPackages.removeDuplicates();
 
-    PackageKit::Transaction * t = PackageKit::Daemon::getPackages();
+    qDebug() << "needed..." << neededPackages.count();
+    PackageKit::Transaction * t = PackageKit::Daemon::resolve(neededPackages);
     connect(t, &PackageKit::Transaction::finished, this, &PackageKitBackend::getPackagesFinished);
     connect(t, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackage);
     connect(t, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-    acquireFetching(true);
 
     fetchUpdates();
 }
@@ -139,7 +154,6 @@ void PackageKitBackend::fetchUpdates()
     connect(tUpdates, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesFinished);
     connect(tUpdates, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageToUpdate);
     connect(tUpdates, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-    acquireFetching(true);
     m_updatesPackageId.clear();
 }
 
@@ -147,33 +161,46 @@ void PackageKitBackend::fetchUpdates()
 void PackageKitBackend::addPackage(PackageKit::Transaction::Info info, const QString &packageId, const QString &summary)
 {
     const QString packageName = PackageKit::Daemon::packageName(packageId);
-    QVector<AbstractResource*> r = resourcesByPackageName(packageName, true);
+    QSet<AbstractResource*> r = resourcesByPackageName(packageName);
     if (r.isEmpty()) {
-        r += new PackageKitResource(packageName, summary, this);
-        m_updatingPackages.packages[packageName] = r.last();
+        auto pk = new PackageKitResource(packageName, summary, this);
+        r = { pk };
+        m_packagesToAdd.insert(pk);
     }
     foreach(auto res, r)
-        static_cast<PackageKitResource*>(res)->addPackageId(info, packageId, summary);
+        static_cast<PackageKitResource*>(res)->addPackageId(info, packageId);
 }
 
 void PackageKitBackend::getPackagesFinished(PackageKit::Transaction::Exit exit)
 {
-    Q_ASSERT(m_isFetching);
-
     if (exit != PackageKit::Transaction::ExitSuccess) {
         qWarning() << "error while fetching details" << exit;
     }
 
-    for(auto it = m_updatingPackages.packages.begin(); it != m_updatingPackages.packages.end(); ) {
+    for(auto it = m_packages.packages.cbegin(); it != m_packages.packages.cend(); ++it) {
         auto pkr = qobject_cast<PackageKitResource*>(it.value());
         if (pkr->packages().isEmpty()) {
             qWarning() << "Failed to find package for" << it.key();
-            it.value()->deleteLater();
-            it = m_updatingPackages.packages.erase(it);
-        } else
-            ++it;
+            m_packagesToDelete += pkr;
+        }
     }
+    includePackagesToAdd();
+}
 
+void PackageKitBackend::includePackagesToAdd()
+{
+    if (m_packagesToAdd.isEmpty() && m_packagesToDelete.isEmpty())
+        return;
+
+    acquireFetching(true);
+    foreach(PackageKitResource* res, m_packagesToAdd) {
+        m_packages.packages[res->packageName()] = res;
+    }
+    foreach(PackageKitResource* res, m_packagesToDelete) {
+        m_packages.packages.take(res->packageName())->deleteLater();
+    }
+    m_packagesToAdd.clear();
+    m_packagesToDelete.clear();
     acquireFetching(false);
 }
 
@@ -184,22 +211,22 @@ void PackageKitBackend::transactionError(PackageKit::Transaction::Error, const Q
 
 void PackageKitBackend::packageDetails(const PackageKit::Details& details)
 {
-    QVector<AbstractResource*> resources = resourcesByPackageName(PackageKit::Daemon::packageName(details.packageId()), false);
-    foreach(AbstractResource* res, resources)
+    const QSet<AbstractResource*> resources = resourcesByPackageName(PackageKit::Daemon::packageName(details.packageId()));
+    if (resources.isEmpty())
+        qWarning() << "couldn't find package for" << details.packageId();
+
+    foreach(AbstractResource* res, resources) {
         qobject_cast<PackageKitResource*>(res)->setDetails(details);
+    }
 }
 
-QVector<AbstractResource*> PackageKitBackend::resourcesByPackageName(const QString& name, bool updating) const
+QSet<AbstractResource*> PackageKitBackend::resourcesByPackageName(const QString& name) const
 {
-    const Packages * const f = (updating ? &m_updatingPackages : &m_packages);
-    const QHash<QString, QStringList> *dictionary = &f->packageToApp;
-    const QHash<QString, AbstractResource*> *pkgs = &f->packages;
-
-    const QStringList names = dictionary->value(name, QStringList(name));
-    QVector<AbstractResource*> ret;
+    const QStringList names = m_packages.packageToApp.value(name, QStringList(name));
+    QSet<AbstractResource*> ret;
     ret.reserve(names.size());
     foreach(const QString& name, names) {
-        AbstractResource* res = pkgs->value(name);
+        AbstractResource* res = m_packages.packages.value(name);
         if (res)
             ret += res;
     }
@@ -209,8 +236,13 @@ QVector<AbstractResource*> PackageKitBackend::resourcesByPackageName(const QStri
 void PackageKitBackend::refreshDatabase()
 {
     if (!m_refresher) {
+        acquireFetching(true);
         m_refresher = PackageKit::Daemon::refreshCache(false);
-        connect(m_refresher.data(), &PackageKit::Transaction::finished, this, &PackageKitBackend::reloadPackageList);
+        connect(m_refresher.data(), &PackageKit::Transaction::finished, this, [this]() {
+            reloadPackageList();
+            acquireFetching(false);
+            delete m_refresher;
+        });
     } else {
         qWarning() << "already resetting";
     }
@@ -218,7 +250,7 @@ void PackageKitBackend::refreshDatabase()
 
 QVector<AbstractResource*> PackageKitBackend::allResources() const
 {
-    return m_packages.packages.values().toVector();
+    return containerValues<QVector<AbstractResource*>>(m_packages.packages);
 }
 
 AbstractResource* PackageKitBackend::resourceByPackageName(const QString& name) const
@@ -233,6 +265,13 @@ QList<AbstractResource*> PackageKitBackend::searchPackageName(const QString& sea
     Q_FOREACH (AbstractResource* res, m_packages.packages) {
         if (res->name().contains(searchText, Qt::CaseInsensitive)) {
             ret += res;
+        } else {
+            foreach(const QString &mime, res->mimetypes()) {
+                if (mime.contains(searchText, Qt::CaseInsensitive)) {
+                    ret += res;
+                    break;
+                }
+            }
         }
     }
     return ret;
@@ -298,39 +337,25 @@ void PackageKitBackend::installApplication(AbstractResource* app)
     addTransaction(new PKTransaction({app}, Transaction::InstallRole));
 }
 
-void PackageKitBackend::cancelTransaction(AbstractResource* app)
-{
-    Q_FOREACH (Transaction* t, m_transactions) {
-        PKTransaction* pkt = qobject_cast<PKTransaction*>(t);
-        if (pkt->resource() == app) {
-            if (pkt->transaction()->allowCancel()) {
-                pkt->transaction()->cancel();
-            } else {
-                qWarning() << "trying to cancel a non-cancellable transaction: " << app->name();
-            }
-            break;
-        }
-    }
-}
-
 void PackageKitBackend::removeApplication(AbstractResource* app)
 {
     Q_ASSERT(!isFetching());
     addTransaction(new PKTransaction({app}, Transaction::RemoveRole));
 }
 
-QList<AbstractResource*> PackageKitBackend::upgradeablePackages() const
+QSet<AbstractResource*> PackageKitBackend::upgradeablePackages() const
 {
-    QVector<AbstractResource*> ret;
+    QSet<AbstractResource*> ret;
+    ret.reserve(m_updatesPackageId.size());
     Q_FOREACH (const QString& pkgid, m_updatesPackageId) {
         const QString pkgname = PackageKit::Daemon::packageName(pkgid);
-        const auto pkgs = resourcesByPackageName(pkgname, false);
+        const auto pkgs = resourcesByPackageName(pkgname);
         if (pkgs.isEmpty()) {
             qWarning() << "couldn't find resource for" << pkgid;
         }
-        ret += pkgs;
+        ret.unite(pkgs);
     }
-    return ret.toList();
+    return ret;
 }
 
 void PackageKitBackend::addPackageToUpdate(PackageKit::Transaction::Info info, const QString& packageId, const QString& summary)
@@ -344,14 +369,13 @@ void PackageKitBackend::addPackageToUpdate(PackageKit::Transaction::Info info, c
 void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
 {
     if (!m_updatesPackageId.isEmpty()) {
-        acquireFetching(true);
         PackageKit::Transaction* transaction = PackageKit::Daemon::getDetails(m_updatesPackageId.toList());
         connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
         connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
         connect(transaction, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesDetailsFinished);
     }
 
-    acquireFetching(false);
+    includePackagesToAdd();
     emit updatesCountChanged();
 }
 
@@ -360,7 +384,6 @@ void PackageKitBackend::getUpdatesDetailsFinished(PackageKit::Transaction::Exit
     if (exit != PackageKit::Transaction::ExitSuccess) {
         qWarning() << "Couldn't figure out the updates on PackageKit backend" << exit;
     }
-    acquireFetching(false);
 }
 
 bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource* res) const
@@ -371,7 +394,7 @@ bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource* res)
 QString PackageKitBackend::upgradeablePackageId(const PackageKitResource* res) const
 {
     QString name = res->packageName();
-    for (const QString& pkgid: m_updatesPackageId) {
+    foreach (const QString& pkgid, m_updatesPackageId) {
         if (PackageKit::Daemon::packageName(pkgid) == name)
             return pkgid;
     }
