@@ -35,7 +35,6 @@ class DISCOVERCOMMON_EXPORT ResourcesModel : public QAbstractListModel
     Q_OBJECT
     Q_PROPERTY(int updatesCount READ updatesCount NOTIFY updatesCountChanged)
     Q_PROPERTY(bool isFetching READ isFetching NOTIFY fetchingChanged)
-    Q_PROPERTY(QVariantList messageActions READ messageActionsVariant NOTIFY fetchingChanged)
     public:
         enum Roles {
             NameRole = Qt::UserRole,
@@ -81,15 +80,15 @@ class DISCOVERCOMMON_EXPORT ResourcesModel : public QAbstractListModel
         bool isBusy() const;
         bool isFetching() const;
         QList<QAction*> messageActions() const;
-        QVariantList messageActionsVariant() const;
         
         QHash<int, QByteArray> roleNames() const override;
 
+        Q_SCRIPTABLE bool isExtended(const QString &id);
+
     public Q_SLOTS:
         void installApplication(AbstractResource* app, const AddonList& addons);
         void installApplication(AbstractResource* app);
         void removeApplication(AbstractResource* app);
-        void cancelTransaction(AbstractResource* app);
 
     Q_SIGNALS:
         void fetchingChanged();
@@ -102,11 +101,13 @@ class DISCOVERCOMMON_EXPORT ResourcesModel : public QAbstractListModel
         void resetBackend(AbstractResourcesBackend* backend);
         void cleanBackend(AbstractResourcesBackend* backend);
         void callerFetchingChanged();
-        void updateCaller();
+        void updateCaller(const QVector<QByteArray>& properties);
         void registerAllBackends();
         void resourceChangedByTransaction(Transaction* t);
+        void emitResourceChanges(AbstractResource* res, const QVector<QByteArray> &properties);
 
     private:
+        QVector<int> propertiesToRoles(const QVector<QByteArray>& propertyNames) const;
         int rowsBeforeBackend(AbstractResourcesBackend* backend, QVector<QVector<AbstractResource*>>::iterator& backendsResources);
 
         ///@p initialize tells if all backends load will be triggered on construction
